<?php
namespace Geral\Logic\Projeto;

use \Geral\Util\NovoCodigo;
use \Geral\Util\Ambiente;
use \Geral\Util\Util;

use \Setforms2\Runtime\Helper\Procedure;
use \Setforms2\Runtime\Helper\Procedure\Param;

/**
 * Classe para tratamento dos valores necessários à manutenção 
 * da entidade PROJETO.
 *
 * @author newtonsantos
 */
class Projeto {

    public static function verificaTipoManutencao($nrorg, $cdfilial, $cdprojeto, $idsitprojeto, $nrbaselinepr, $cdvercronprj) {
        $blockedit = false;
        $blockchanges = !(strpos('CSE', $idsitprojeto) === false);
        
        $cronoprojetoRepository = \Teknisa\Entities\Cronoprojeto::getRepository();
            
        if(!$blockchanges) {
            $cronoprojeto = $cronoprojetoRepository->consultaPercConclusaoCronograma($nrorg, $cdfilial, $cdprojeto, $nrbaselinepr, $cdvercronprj);
            $vrperconcprj = (count($cronoprojeto) > 0) ? $cronoprojeto[0]['VRPERCONCPRJ'] : 0;            
            
            $blockchanges = ($vrperconcprj == 100);
        }

        if(!$blockchanges) {
            $cronoprojeto = $cronoprojetoRepository->consultaMaxVersaoCronograma($nrorg, $cdfilial, $cdprojeto, $nrbaselinepr);
            $cdmaxvercron = (count($cronoprojeto) > 0) ? $cronoprojeto[0]['CDVERCRONPRJ'] : 0;            
            
            $blockchanges = ($cdvercronprj != $cdmaxvercron);
        }

        if(!$blockchanges) {
            $cdnivelitprj = '3'; /* Filtra apenas as linhas correspondentes às atividades */
            $where = 'CDFILIAL = ? AND CDPROJETO = ? AND NRBASELINEPR = ? AND CDVERCRONPRJ = ? AND CDNIVELITPRJ = ? AND NRORG = ? AND NVL(VRPERCONCPRJ, 0) <> NVL(VRPERCONCALC, 0)';
            $params = array($cdfilial, $cdprojeto, $nrbaselinepr, $cdvercronprj, $cdnivelitprj, $nrorg);
            $lstcronoprojeto = \Teknisa\Entities\Cronoprojeto::getObjectsByWhere($where, $params);
        
            if(count($lstcronoprojeto) > 0) {
                $blockedit = true;
            }
        }
        
        $tipomanutencao = array(
            "edit"   => $blockedit,
            "change" => $blockchanges
        );
        
        return $tipomanutencao;
    }

    public static function getGrupoProjetoImplantacao() {
        
        /* O grupo de projeto referente à implantação é fixo por enquanto.
           quando for necessário parametrizar, bastará alterar essa rotina. */
        $grupoprojeto = '0000000005';
        
        return $grupoprojeto;
    }

    public static function getBaselineAtualGrupoProjeto($nrorg, $grupoprojeto, $dataproj) {
        
        $nrbaselinepr = false;
        
        $dataproj = \Geral\Util\DateUtil::truncateString($dataproj);
        $where = "NRORG = ? AND 
                  CDGRUPOPROJ = ? AND 
                  ? BETWEEN DTIPATCHGRPROJ AND DTFPATCHGRPROJ";
        $params = array($nrorg, $grupoprojeto, $dataproj);
        $baselinegrproj = \Teknisa\Entities\Baselinegrproj::getObjectByWhere($where, $params);
        
        if($baselinegrproj instanceof \Teknisa\Entities\Baselinegrproj) {
            $nrbaselinepr = $baselinegrproj->getNrbasegrpr();
        }
        
        return $nrbaselinepr;
    }

    /**
     * Insere uma baseline para o projeto
     *
     * @param integer $nrOrg           Numero da organizacao.
     *
     * @return Baselineproj
     *
     * @author newtonsantos
     */
    public static function insereBaselineProjeto($nrorg, $cdfilial, $cdprojeto, $nrbaselinepr, $cdoperador, $dtprojeto) {

        $idsitbaselin = '1'; /* Pendente */
        $dtprojeto = \Geral\Util\DateUtil::truncateString($dtprojeto);
        $dtincbaselin = \Geral\Util\DateUtil::getDataDeString($dtprojeto);
        
        $connection = \Setforms2\Shared\Registry::getEntityManager();
        try {
            $baselineproj = new \Geral\Entities\Baselineproj();
            $baselineproj->setNrorg($nrorg);
            $baselineproj->setCdfilial($cdfilial);
            $baselineproj->setCdprojeto($cdprojeto);
            $baselineproj->setNrbaselinepr($nrbaselinepr);
            $baselineproj->setIdsitbaselin($idsitbaselin);
            $baselineproj->setCdoperresp($cdoperador);
            $baselineproj->setCdoperincl($cdoperador);
            $baselineproj->setDtincbaselin($dtincbaselin);
            $baselineproj->setDtinclusao($dtincbaselin);
            $baselineproj->persist();
           
            $connection->flush();
        } catch (\Exception $e) {
            throw $e;
        }
            
        return is_object($baselineproj) ? $baselineproj : false;
    }

    /**
     * Insere operador como envolvido no projeto
     *
     * @param integer $nrOrg           Numero da organizacao.
     *
     * @return Operprojeto
     *
     * @author newtonsantos
     */
    public static function insereEnvolvidoProjeto($nrorg, $cdfilial, $cdprojeto, $cdoperador, $cdgrupocargo, $dsresponsop = null) {

        $operProjeto = \Geral\Entities\Operprojeto::findOneByCdfilialAndCdprojetoAndCdoperadorAndNrorg($cdfilial, $cdprojeto, $cdoperador, $nrorg); 
        
        if (!is_object($operProjeto)) {
            $dtatual = \Geral\Util\DateUtil::getDataAtual();

            $connection = \Setforms2\Shared\Registry::getEntityManager();
            try {
                $operProjeto = new \Geral\Entities\Operprojeto();
                $operProjeto->setNrorg($nrorg);
                $operProjeto->setCdfilial($cdfilial);
                $operProjeto->setCdprojeto($cdprojeto);
                $operProjeto->setCdoperador($cdoperador);
                $operProjeto->setCdgrupocargo($cdgrupocargo);
                $operProjeto->setDsresponsop($dsresponsop);
                $operProjeto->setDtincloperproj($dtatual);
                $operProjeto->setDtinclusao($dtatual);
                $operProjeto->persist();

                $connection->flush();
            } catch (\Exception $e) {
                throw $e;
            }
        }
            
        return is_object($operProjeto) ? $operProjeto : false;
    }

    /**
     * Consulta próxima fila do cronograma
     *
     * @param integer $nrOrg           Numero da organizacao.
     *
     * @return Cronoprojeto
     *
     * @author newtonsantos
     */
    public static function getProxFilaCronograma($nrorg, $cdfilial, $cdprojeto, $nrbaselinepr, $cdvercronprj) {
        $cronoprojetoRepository = \Teknisa\Entities\Cronoprojeto::getRepository();
        $cronoprojeto = $cronoprojetoRepository->consultaProxFilaCronograma($nrorg, $cdfilial, $cdprojeto, $nrbaselinepr, $cdvercronprj);

        return $cronoprojeto;
    }
    
    /**
     * Consulta a versão atual do cronograma (maior versão)
     *
     * @param integer $nrOrg           Numero da organizacao.
     *
     * @return Cronoprojeto
     *
     * @author newtonsantos
     */
    public static function getMaxVersaoCronograma($nrorg, $cdfilial, $cdprojeto, $nrbaselinepr) {
        $cronoprojetoRepository = \Teknisa\Entities\Cronoprojeto::getRepository();
        $cronoprojeto = $cronoprojetoRepository->consultaMaxVersaoCronograma($nrorg, $cdfilial, $cdprojeto, $nrbaselinepr);
        
        return $cronoprojeto;
    }    
            
    /**
     * Insere um ítem de cronograma
     *
     * @param integer $nrOrg           Numero da organizacao.
     *
     * @return Cronoprojeto
     *
     * @author newtonsantos
     */
    public static function insereItemCronogramaProjeto($nrorg, $cdfilial, $cdprojeto, $nrbaselinepr, $cdvercronprj, $cdoperador, $dtprojeto, $dsitcronoprj, $idnivelitcrono, $botransaction = false, $cdcliente = NULL, $nrlicencauso = NULL, $nrsoliserv = NULL, $nrtarefa = NULL, $dadosatividade =  NULL) {

        $cronoprojetoRepository = \Teknisa\Entities\Cronoprojeto::getRepository();
        $cronoprojeto = $cronoprojetoRepository->consultaProxSequencialCronograma($nrorg, $cdfilial, $cdprojeto, $nrbaselinepr, $cdvercronprj);

        $cdseqcronprj = $cronoprojeto[0]['CDSEQCRONPRJ'];
        $dtprojeto = \Geral\Util\DateUtil::truncateString($dtprojeto);
        $dtcronogproj = \Geral\Util\DateUtil::getDataDeString($dtprojeto);
        $dtatual = \Geral\Util\DateUtil::getDataAtual();
        $cdoperatual = \Geral\Util\Ambiente::getCdOperador();

        $idmarcoproj = 'N';
        
        $nrplativtare = null;
        $cdfilaprio = null;
        $cdprecitcrpj = null;
        $qtdiafolgacron = null;
        $nrdiasitcrpj = null;
            
        $connection = \Setforms2\Shared\Registry::getEntityManager();
        if($botransaction){
            $connection->beginTransaction();
        }
        
        switch ($idnivelitcrono) {
            case 'PROJ':
                $cdnivelitprj = '1';
                break;
            case 'SOLI':
                $cdnivelitprj = '2';
                $soliserv = self::insereSolicitacaoServico($nrorg, $cdfilial, $cdoperador, $cdprojeto, $nrbaselinepr, $dsitcronoprj, $cdcliente, $nrlicencauso);
                if ($soliserv) {
                    $nrsoliserv = $soliserv->getNrsoliserv();
                }
                break;
            case 'TARE':
                $cdnivelitprj = 'T';
                $tarefa = self::insereTarefa($nrorg, $cdfilial, $cdoperador, $dsitcronoprj, $nrsoliserv);
                if ($tarefa) {
                    $nrtarefa = $tarefa->getNrtarefa();
                }
                break;
            case 'ATIV':
                $cdnivelitprj = '3';
                $planativtare = self::insereAtividade($nrorg, $cdfilial, $cdoperador, $dsitcronoprj, $nrsoliserv, $nrtarefa, $dadosatividade);
                if ($planativtare) {
                    $nrplativtare = $planativtare->getNrplativtare();
                    
                    $cdfilaprio = $dadosatividade["CDFILAPRIO"];
                    $cdprecitcrpj = $dadosatividade["CDPRECITCRPJ"];
                    $qtdiafolgacron = $dadosatividade["QTDIAFOLGACRON"];
                    $nrdiasitcrpj = $dadosatividade["NRDIASITCRPJ"];
                    if($nrdiasitcrpj == 0) {
                        $nrdiasitcrpj = $planativtare->getQttemplaitar();
                    }

                    $cdgrupocargo = $dadosatividade["CDGRUPOCARGO"];
                    self::insereEnvolvidoProjeto($nrorg, $cdfilial, $cdprojeto, $cdoperador, $cdgrupocargo);
                }
                break;
        } 
        
        try {
            $cronoProjeto = new \Teknisa\Entities\Cronoprojeto();
            $cronoProjeto->setNrorg($nrorg);
            $cronoProjeto->setCdfilial($cdfilial);
            $cronoProjeto->setCdprojeto($cdprojeto);
            $cronoProjeto->setNrbaselinepr($nrbaselinepr);
            $cronoProjeto->setCdvercronprj($cdvercronprj);
            $cronoProjeto->setCdseqcronprj($cdseqcronprj);
            $cronoProjeto->setNrsoliserv($nrsoliserv);
            $cronoProjeto->setNrtarefa($nrtarefa);
            $cronoProjeto->setNrplativtare($nrplativtare);
            $cronoProjeto->setDsitcronoprj($dsitcronoprj);
            $cronoProjeto->setCdnivelitprj($cdnivelitprj);
            $cronoProjeto->setCdfilaprio($cdfilaprio);
            $cronoProjeto->setCdprecitcrpj($cdprecitcrpj);
            $cronoProjeto->setQtdiafolgacron($qtdiafolgacron);
            $cronoProjeto->setNrdiasitcrpj($nrdiasitcrpj);
            $cronoProjeto->setCdoperador($cdoperador);
            $cronoProjeto->setIdmarcoproj($idmarcoproj);
            $cronoProjeto->setDtinclusao($dtatual);
            $cronoProjeto->setDtultatu($dtatual);
            $cronoProjeto->setNrorginclusao($nrorg);
            $cronoProjeto->setCdoperinclusao($cdoperatual);
            $cronoProjeto->setNrorgultatu($nrorg);
            $cronoProjeto->setCdoperultatu($cdoperatual);
            $cronoProjeto->setDtcronogproj($dtcronogproj);
            $cronoProjeto->persist();
           
            $connection->flush();
            
            if($botransaction){
                $connection->commit();
            }
        } catch (\Exception $e) {
            if($botransaction){
                $connection->rollback();
            }
            throw $e;
        }
            
        return is_object($cronoProjeto) ? $cronoProjeto : false;
    }

    /**
     * retorna um sequencial para SOLISERV.
     *
     * @return string
     *
     * @author newtonsantos - 19/07/2013
     */
    public static function geraNovoCodigoSoliserv($cdfilial) {
        return Novocodigo::geraCodigo(Ambiente::getNrorg(), 'SOLISERV'.$cdfilial);
    }

    /**
     * retorna o assunto que deve ser utilizado para solicitações abertas
     * através da inclusão de ítens de cronograma.
     *
     * @return string
     *
     * @author newtonsantos - 19/07/2013
     */
    public static function getAssuntoSoliServ() {
        
        /* O assunto referente à solicitação de serviço criada como ítem de 
           cronograma é fixo por enquanto. Quando for necessário parametrizar, 
           bastará alterar essa rotina. */
        $cdassunto = '00580';
        
        return $cdassunto;
    }

    /**
     * retorna o assunto que deve ser utilizado para solicitações abertas
     * através da inclusão de ítens de cronograma.
     *
     * @return string
     *
     * @author newtonsantos - 19/07/2013
     */
    
    public static function getProdutoSoliServ() {
        
        /* O produto referente à solicitação de serviço criada como ítem de 
           cronograma é fixo por enquanto. Quando for necessário parametrizar, 
           bastará alterar essa rotina. */
        $cdproduto = '0000001429';
        
        return $cdproduto;
    }

    public static function getSubprodutoSoliServ() {
        
        /* O subproduto referente à solicitação de serviço criada como ítem de 
           cronograma é fixo por enquanto. Quando for necessário parametrizar, 
           bastará alterar essa rotina. */
        $cdproduto = '0000001410';
        
        return $cdproduto;
    }

    /**
     * Insere uma solicitação de serviço
     *
     * @param integer $nrOrg           Numero da organizacao.
     *
     * @return Soliserv
     *
     * @author newtonsantos
     */
    public static function insereSolicitacaoServico($nrorg, $cdfilial, $cdoperador, $cdprojeto, $nrbaselinepr, $dsrefesoli, $cdcliente, $nrlicencauso) {

        $nrsoliserv = self::geraNovoCodigoSoliserv($cdfilial);
        $nrpriosoli = '000';
        $cdassunto = self::getAssuntoSoliServ();
        $cdprodcomp = self::getProdutoSoliServ();
        $cdprodgrup = self::getSubprodutoSoliServ();
        
        $paramgeral = \Geral\Entities\Paramgeral::findOneByNrorg($nrorg);
        $cdfasetram = $paramgeral->getCdfasetram();
        $txacaolftram = tk_get_message_by_name('msg_win_gep00001_000_FaseDeAberturaSolicitacao');
        $dssoliserv = tk_get_message_by_name('msg_win_gep00001_000_DescricaoAberturaSolicitacao', array($cdprojeto, $dsrefesoli));
        
        $idstatussoli = 'P';
        $idsolience = 'N';
        $idsoliorca = 'N';
        $idresdispver = 'N';
        $idsolilida = 'N';
        $idsolctrlsla = 'N';
        
        $dtatual = \Geral\Util\DateUtil::getDataAtual();
        $cdoperatual = \Geral\Util\Ambiente::getCdOperador();
        
        $connection = \Setforms2\Shared\Registry::getEntityManager();
        try {
            $soliServ = new \Faturamento\Entities\Soliserv();
            $soliServ->setNrorg($nrorg);
            $soliServ->setCdfilial($cdfilial);
            $soliServ->setNrsoliserv($nrsoliserv);
            $soliServ->setNrpriosoli($nrpriosoli);
            $soliServ->setDsrefesoli($dsrefesoli);
            $soliServ->setCdassunto($cdassunto);
            $soliServ->setCdoperador($cdoperador);
            $soliServ->setCdprodcomp($cdprodcomp);
            $soliServ->setCdprodgrup($cdprodgrup);
            $soliServ->setCdcliente($cdcliente);
            $soliServ->setCdfasetram($cdfasetram);
            $soliServ->setDtabersoli($dtatual);
            $soliServ->setIdstatussoli($idstatussoli);
            $soliServ->setIdsolience($idsolience);
            $soliServ->setIdsoliorca($idsoliorca);
            $soliServ->setNrlicencauso($nrlicencauso);
            $soliServ->setIdresdispver($idresdispver);
            $soliServ->setCdopersoli($cdoperador);
            $soliServ->setCdoperresp($cdoperador);
            $soliServ->setIdsolilida($idsolilida);
            $soliServ->setIdsolctrlsla($idsolctrlsla);
            $soliServ->setDtinclusao($dtatual);
            $soliServ->setDtultatu($dtatual);
            $soliServ->setNrorginclusao($nrorg);
            $soliServ->setCdoperinclusao($cdoperatual);
            $soliServ->setNrorgultatu($nrorg);
            $soliServ->setCdoperultatu($cdoperatual);
            $soliServ->persist();
            $connection->flush();
            $connection->refresh($soliServ);
            
            $cdfasetram = $soliServ->getCdfasetram();
            $cdoperresp = $soliServ->getCdoperresp();
            
            $logFaseTram = new \Teknisa\Entities\Logfasetram();
            $logFaseTram->setNrorg($nrorg);
            $logFaseTram->setCdfilial($cdfilial);
            $logFaseTram->setNrsoliserv($nrsoliserv);
            $logFaseTram->setDtalfasetram($dtatual);
            $logFaseTram->setCdfasetram($cdfasetram);
            $logFaseTram->setCdoperador($cdoperador);
            $logFaseTram->setCdoperresp($cdoperresp);
            $logFaseTram->setTxacaolftram($txacaolftram);
            $logFaseTram->setDtinclusao($dtatual);
            $logFaseTram->setDtultatu($dtatual);
            $logFaseTram->setNrorginclusao($nrorg);
            $logFaseTram->setCdoperinclusao($cdoperatual);
            $logFaseTram->setNrorgultatu($nrorg);
            $logFaseTram->setCdoperultatu($cdoperatual);
            $logFaseTram->persist();
            
            $descSoli = new \Teknisa\Entities\Descsoli();
            $descSoli->setNrorg($nrorg);
            $descSoli->setCdfilial($cdfilial);
            $descSoli->setNrsoliserv($nrsoliserv);
            $descSoli->setDssoliserv($dssoliserv);
            $descSoli->setDtinclusao($dtatual);
            $descSoli->setDtultatu($dtatual);
            $descSoli->setNrorginclusao($nrorg);
            $descSoli->setCdoperinclusao($cdoperatual);
            $descSoli->setNrorgultatu($nrorg);
            $descSoli->setCdoperultatu($cdoperatual);
            $descSoli->persist();
            
            $idsitusolbaseli = 'I';
            
            $soliBaseLine = new \Teknisa\Entities\Solibaseline();
            $soliBaseLine->setNrorg($nrorg);
            $soliBaseLine->setCdfilial($cdfilial);
            $soliBaseLine->setCdprojeto($cdprojeto);
            $soliBaseLine->setNrbaselinepr($nrbaselinepr);
            $soliBaseLine->setNrsoliserv($nrsoliserv);
            $soliBaseLine->setDtinsolbaselipr($dtatual);
            $soliBaseLine->setCdopinsolbaseli($cdoperatual);
            $soliBaseLine->setIdsitusolbaseli($idsitusolbaseli);
            $soliBaseLine->setDtinclusao($dtatual);
            $soliBaseLine->setDtultatu($dtatual);
            $soliBaseLine->setNrorginclusao($nrorg);
            $soliBaseLine->setCdoperinclusao($cdoperatual);
            $soliBaseLine->setNrorgultatu($nrorg);
            $soliBaseLine->setCdoperultatu($cdoperatual);
            $soliBaseLine->persist();
            
            $connection->flush();
        } catch (\Exception $e) {
            throw $e;
        }
            
        return is_object($soliServ) ? $soliServ : false;
    }

    /**
     * retorna um sequencial para TAREFA.
     *
     * @return string
     *
     * @author newtonsantos - 06/08/2013
     */
    public static function geraNovoCodigoTarefa($cdfilial) {
        return Novocodigo::geraCodigo(Ambiente::getNrorg(), 'TAREFA'.$cdfilial);
    }

    /**
     * Insere uma tarefa
     *
     * @param integer $nrOrg           Numero da organizacao.
     *
     * @return Tarefa
     *
     * @author newtonsantos
     */
    public static function insereTarefa($nrorg, $cdfilial, $cdoperador, $dsrefetarefa, $nrsoliserv) {

        $nrtarefa = self::geraNovoCodigoTarefa($cdfilial);

        $soliserv = \Faturamento\Entities\Soliserv::findOneByCdfilialAndNrsoliservAndNrorg($cdfilial, $nrsoliserv, $nrorg);
        $cdassunto = $soliserv->getCdassunto();

        $dtatual = \Geral\Util\DateUtil::getDataAtual();
        $cdoperatual = \Geral\Util\Ambiente::getCdOperador();

        $idsittarefa = 'P';
        $nrpriotarefa = '000';
        $idorigtare = '1';
        
        $connection = \Setforms2\Shared\Registry::getEntityManager();
        try {
            $tarefa = new \Geral\Entities\Tarefa();
            $tarefa->setNrorg($nrorg);
            $tarefa->setCdfilial($cdfilial);
            $tarefa->setNrtarefa($nrtarefa);
            $tarefa->setDsrefetarefa($dsrefetarefa);
            $tarefa->setNrsoliserv($nrsoliserv);
            $tarefa->setCdassunto($cdassunto);
            $tarefa->setCdoperresp($cdoperador);
            $tarefa->setCdoperaber($cdoperatual);
            $tarefa->setDtabertarefa($dtatual);
            $tarefa->setIdsittarefa($idsittarefa);
            $tarefa->setNrpriotarefa($nrpriotarefa);
            $tarefa->setIdorigtare($idorigtare);
            $tarefa->setDtinclusao($dtatual);
            $tarefa->setDtultatu($dtatual);
            $tarefa->setNrorginclusao($nrorg);
            $tarefa->setCdoperinclusao($cdoperatual);
            $tarefa->setNrorgultatu($nrorg);
            $tarefa->setCdoperultatu($cdoperatual);
            $tarefa->persist();
            
            $connection->flush();
        } catch (\Exception $e) {
            throw $e;
        }
            
        return is_object($tarefa) ? $tarefa : false;
    }

    /**
     * retorna um sequencial para PLANATIVTARE.
     *
     * @return string
     *
     * @author newtonsantos - 06/08/2013
     */
    public static function geraNovoCodigoPlanativtare($cdfilial, $nrtarefa) {
        return Novocodigo::geraCodigo(Ambiente::getNrorg(), 'PLANATIVTARE'.$cdfilial.$nrtarefa, 1, 5);
    }

    /**
     * retorna o número correspondente à complexidade da atividade.
     *
     * @return Array 
     *
     * @author newtonsantos - 07/08/2013
     */
    public static function getComplexAtividade($nrcplxittrab, $vrfatocplx, $vrprodmedes, $qttempminativ, $vrpesoativgrc) {
        $vrcomplexidade = $nrcplxittrab * $vrfatocplx;
        $qttemplaitar = (($vrcomplexidade / $vrprodmedes) * $vrpesoativgrc) + $qttempminativ;
        if ($qttempminativ > 0) {
            $vrcomplexidade = round($qttemplaitar * $vrprodmedes,2);
        }
        
        $Complexidade = array(
            "nrcplx" => $vrcomplexidade,
            "qttemp" => $qttemplaitar
        );

        return $Complexidade;
    }

    /**
     * retorna o peso da atividade atribuída ao grupo de cargo ligada ao cargo do operador.
     *
     * @return float
     *
     * @author newtonsantos - 07/08/2013
     */
    public static function getGrupoCargoOperador($nrorg, $cdoperador) {
        $operador = \Geral\Entities\Operador::findOneByCdoperadorAndNrorg($cdoperador, $nrorg); 
        $cargo = \Geral\Entities\Cargo::findOneByCdcargoAndNrorg($operador->getCdcargo(), $nrorg); 
        $grupocargo = \Geral\Entities\GrupoCargo::findOneByCdgrupocargoAndNrorg($cargo->getCdgrupocargo(), $nrorg); 

        if(is_object($grupocargo) && $grupocargo->GetVrpesoativgrc() != null) {
            $vrpesoativgrc = ((100 + $grupocargo->GetVrpesoativgrc()) / 100);
        } else {
            $vrpesoativgrc = 100;
        }
        
        $grupocargooper = array(
            "cdgrupocargo" => $cargo->getCdgrupocargo(),
            "vrpesoativgrc" => $vrpesoativgrc
        );
        
        return $grupocargooper;
    }

    /**
     * Insere uma atividade
     *
     * @param integer $nrOrg           Numero da organizacao.
     *
     * @return Planativtare
     *
     * @author newtonsantos
     */
    public static function insereAtividade($nrorg, $cdfilial, $cdoperador, $dsitcronoprj, $nrsoliserv, $nrtarefa, $dadosatividade) {

        $itemtarefaRepository = \Teknisa\Entities\Itemtarefa::getRepository();
        $ittarefa = $itemtarefaRepository->consultaProxSequencialItemTarefa($nrorg, $cdfilial, $nrtarefa);
        $nrsequittare = $ittarefa[0]['NRSEQUITTARE'];
        
        $dsrefetarefa = $dadosatividade["DSREFETAREFA"];
        $cdoperesprj = $dadosatividade["CDOPERESPRJ"];
        $cditemtrab = $dadosatividade["CDITEMTRAB"];
        $nrcplxittrab = $dadosatividade["NRCPLXITTRAB"];
        $nrdiasitcrpj = $dadosatividade["NRDIASITCRPJ"];

        $nrprioittare = '000';
        
        $dtatual = \Geral\Util\DateUtil::getDataAtual();
        $cdoperatual = \Geral\Util\Ambiente::getCdOperador();

        $connection = \Setforms2\Shared\Registry::getEntityManager();
        try {
            $itemtarefa = new \Teknisa\Entities\Itemtarefa();
            $itemtarefa->setNrorg($nrorg);
            $itemtarefa->setCdfilial($cdfilial);
            $itemtarefa->setNrtarefa($nrtarefa);
            $itemtarefa->setNrsequittare($nrsequittare);
            $itemtarefa->setDsrefeittare($dsitcronoprj);
            $itemtarefa->setDtinclittare($dtatual);
            $itemtarefa->setCdoperador($cdoperador);
            $itemtarefa->setTxplanittare($dsrefetarefa);
            $itemtarefa->setCdoperresp($cdoperesprj);
            $itemtarefa->setNrprioittare($nrprioittare);
            $itemtarefa->setCditemtrab($cditemtrab);
            $itemtarefa->setNrcplxittare($nrcplxittrab);
            $itemtarefa->setNrcplxitemtb($nrcplxittrab);
            $itemtarefa->setDtinclusao($dtatual);
            $itemtarefa->setDtultatu($dtatual);
            $itemtarefa->setNrorginclusao($nrorg);
            $itemtarefa->setCdoperinclusao($cdoperatual);
            $itemtarefa->setNrorgultatu($nrorg);
            $itemtarefa->setCdoperultatu($cdoperatual);
            $itemtarefa->persist();
            
            $nrplativtare = self::geraNovoCodigoPlanativtare($cdfilial, $nrtarefa);
            $cdprocesso = $dadosatividade["CDPROCESSO"];
           
            $complexidade = self::getComplexAtividade($itemtarefa->getNrcplxittare(), $dadosatividade["VRFATOCPLX"], $dadosatividade["VRPRODMEDES"], $dadosatividade["QTTEMPMINATIV"], $dadosatividade["VRPESOATIVGRC"]);
            $nrcplxitativ = $complexidade["nrcplx"];
            $nrcplxplanat = $nrcplxitativ;
            $qttemplaitar = $complexidade["qttemp"];
            if(($nrdiasitcrpj != 0) && ($nrdiasitcrpj != $qttemplaitar)) {
                $qttemplaitar = $nrdiasitcrpj;
                $idatvtempaltcr = 'S';
            } else {
                $qttemplaitar = $complexidade["qttemp"];
                $idatvtempaltcr = 'N';
            }
            
            /* Valores DEFAULT - não tratados pelo Doctrine */
            $idativatrasopla = 'N';
            $idoperpareuniao = 'N';
            $idbloqdescat = 'N';
            $idsentcomuat = 'S';
            $idsitativtar = 'P';
            $idaltstatuspend = 'N';
            $idimpiniativtar = 'S';
            $idevolatenfatur = 'S';
            $idativinclcrono = 'N';

            $planativtare = new \Geral\Entities\Planativtare();
            $planativtare->setNrorg($nrorg);
            $planativtare->setCdfilial($cdfilial);
            $planativtare->setNrtarefa($nrtarefa);
            $planativtare->setNrplativtare($nrplativtare);
            $planativtare->setCdprocesso($cdprocesso);
            $planativtare->setCdoperresp($cdoperador);
            $planativtare->setCditemtrab($cditemtrab);
            $planativtare->setDtincativtar($dtatual);
            $planativtare->setCdoperincl($cdoperatual);
            $planativtare->setNrcplxitativ($nrcplxitativ);
            $planativtare->setNrcplxplanat($nrcplxplanat);
            $planativtare->setQttemplaitar($qttemplaitar);
            $planativtare->setIdativatrasopla($idativatrasopla);
            $planativtare->setIdoperpareuniao($idoperpareuniao);
            $planativtare->setIdatvtempaltcr($idatvtempaltcr);
            $planativtare->setIdbloqdescat($idbloqdescat);
            $planativtare->setIdsentcomuat($idsentcomuat);
            $planativtare->setIdsitativtar($idsitativtar);
            $planativtare->setIdaltstatuspend($idaltstatuspend);
            $planativtare->setIdimpiniativtar($idimpiniativtar);
            $planativtare->setIdevolatenfatur($idevolatenfatur);
            $planativtare->setIdativinclcrono($idativinclcrono);
            $planativtare->setDtinclusao($dtatual);
            $planativtare->setDtultatu($dtatual);
            $planativtare->setNrorginclusao($nrorg);
            $planativtare->setCdoperinclusao($cdoperatual);
            $planativtare->setNrorgultatu($nrorg);
            $planativtare->setCdoperultatu($cdoperatual);
            $planativtare->persist();
            
            $platvrelittr = new \Teknisa\Entities\Platvrelittr();
            $platvrelittr->setNrorg($nrorg);
            $platvrelittr->setCdfilial($cdfilial);
            $platvrelittr->setNrtarefa($nrtarefa);
            $platvrelittr->setNrplativtare($nrplativtare);
            $platvrelittr->setNrsequittare($nrsequittare);
            $platvrelittr->setDtinclusao($dtatual);
            $platvrelittr->setDtultatu($dtatual);
            $platvrelittr->setNrorginclusao($nrorg);
            $platvrelittr->setCdoperinclusao($cdoperatual);
            $platvrelittr->setNrorgultatu($nrorg);
            $platvrelittr->setCdoperultatu($cdoperatual);
            $platvrelittr->persist();

            $connection->flush();
        } catch (\Exception $e) {
            throw $e;
        }
            
        return is_object($planativtare) ? $planativtare : false;
    }

    /**
     * Ordena um array passado como parâmetro
     *
     * @param array $arraytosort  Array que deverá ser ordenado.
     *
     * @author newtonsantos
     */
    private static function ordenaArray($arraytosort, $key) {        
        usort($arraytosort, function ($a, $b) use ($key) {
            return strcmp($a[$key], $b[$key]);
        });

        // NEWTON - Função para gravação de ARRAY em disco (para averiguação)
        // file_put_contents("debug.txt", var_export($lstcronoprojeto, true));
        
        return $arraytosort;
    }

    /**
     * Insere atividades automaticamente de acordo com parâmetros informados pelo usuário
     *
     * @param integer $nrOrg           Número da organização.
     *
     * @author newtonsantos
     */
    public static function cadastroAutomaticoAtividade($nrorg, $cdfilial, $cdprojeto, $nrbaselinepr, $cdvercronprj, $cdoperador, $dtprojeto, $cdcliente, $nrlicencauso, $nrsoliserv, $nrtarefa, $dsrefetarefa, $cdarvprocess, $cdarvittrab, $cditemtrab) {
        $botransaction = true;
        
        if ($nrtarefa == 'NOVOCODIGO') {
            
            $idnivelitcrono = 'TARE';
            $dsitcronoprj = $dsrefetarefa;
            
            $cronoProjeto = \Geral\Logic\Projeto\Projeto::insereItemCronogramaProjeto($nrorg, $cdfilial, $cdprojeto, $nrbaselinepr, 
                                                                                      $cdvercronprj, $cdoperador, $dtprojeto, $dsitcronoprj, 
                                                                                      $idnivelitcrono, $botransaction, $cdcliente, $nrlicencauso, 
                                                                                      $nrsoliserv);
            $nrtarefa = $cronoProjeto->getNrtarefa();                    
        }
        
        if ($cditemtrab == 'A') {
            $itemtrabRepository = \Geral\Entities\Itemtrab::getRepository();
            $lstitemtrab = $itemtrabRepository->consultaItemtrabPorArvore($nrorg, $cdarvittrab);
        } else {
            $lstitemtrab = self::ordenaArray($cditemtrab, 'cdarvittrab');
        }
        
        $processoRepository = \Geral\Entities\Processo::getRepository();
        $lstprocesso = $processoRepository->consultaProcessoPorArvore($nrorg, $cdarvprocess);
            
        $idnivelitcrono = 'ATIV';
        $grupocargo = self::getGrupoCargoOperador($nrorg, $cdoperador);

        foreach ($lstitemtrab as $itemtrab) {
            foreach ($lstprocesso as $processo) {
                $nrcplxittrab = $cditemtrab === 'A' ? $itemtrab['NRCPLXITTRAB'] : $itemtrab['nrcplxittrab'];
                $vrfatocplx = $processo['VRFATOCPLX'];
                $vrprodmedes = $processo['VRPRODMEDES'];
                $qttempminativ = $processo['QTTEMPMINATIV'];
                $vrpesoativgrc = $grupocargo['vrpesoativgrc'];
                
                $complexidade = \Geral\Logic\Projeto\Projeto::getComplexAtividade($nrcplxittrab, $vrfatocplx, $vrprodmedes, $qttempminativ, $vrpesoativgrc);
                $nrdiasitcrpj = $complexidade["qttemp"];
                
                $dadosatividade = array(
                            "DSREFETAREFA"   => $dsrefetarefa,
                            "CDITEMTRAB"     => $cditemtrab === 'A' ? $itemtrab['CDITEMTRAB'] : $itemtrab['cditemtrab'],
                            "NRCPLXITTRAB"   => $nrcplxittrab,
                            "CDPROCESSO"     => $processo['CDPROCESSO'],
                            "VRFATOCPLX"     => $vrfatocplx,
                            "QTTEMPMINATIV"  => $qttempminativ,
                            "VRPRODMEDES"    => $vrprodmedes,
                            "VRPESOATIVGRC"  => $vrpesoativgrc,
                            "CDPRECITCRPJ"   => null,
                            "QTDIAFOLGACRON" => null,
                            "NRDIASITCRPJ"   => $nrdiasitcrpj,
                            "CDOPERESPRJ"    => $cdoperador,
                            "CDGRUPOCARGO"   => $grupocargo['cdgrupocargo']
                );
                $dsitcronoprj = $cditemtrab === 'A' ? $itemtrab['NMITEMTRAB'] : $itemtrab['nmitemtrab'];
                
                \Geral\Logic\Projeto\Projeto::insereItemCronogramaProjeto($nrorg, $cdfilial, $cdprojeto, $nrbaselinepr, 
                                                                          $cdvercronprj, $cdoperador, $dtprojeto, $dsitcronoprj, 
                                                                          $idnivelitcrono, $botransaction, $cdcliente, $nrlicencauso, 
                                                                          $nrsoliserv, $nrtarefa, $dadosatividade);

            }
        }
        
        self::organizaFilaCronograma($nrorg, $cdfilial, $cdprojeto, $nrbaselinepr, $cdvercronprj);
    }

    /**
     * Atualiza um ítem de cronograma
     *
     * @param integer $nrOrg           Numero da organizacao.
     *
     * @return Cronoprojeto
     *
     * @author newtonsantos
     */
    public static function atualizaItemCronogramaProjeto($nrorg, $cdfilial, $cdprojeto, $nrbaselinepr, $cdvercronprj, $cdseqcronprj, $dsitcronoprj, $idnivelitcrono, $cdcliente, $nrlicencauso, $cdfilaprio, $nrsoliserv, $nrtarefa = NULL, $dadosatividade = NULL, $onlychangeoper = false) {

        $cdprecitcrpj = null;
        $qtdiafolgacron = null;
        $nrdiasitcrpj = null;
        $cdoperador = null;
            
        $connection = \Setforms2\Shared\Registry::getEntityManager();
        $connection->beginTransaction();
        
        switch ($idnivelitcrono) {
            case 'SOLI':
                self::atualizaSolicitacaoServico($nrorg, $cdfilial, $nrsoliserv, $cdprojeto, $dsitcronoprj, $cdcliente, $nrlicencauso);
                break;
            case 'TARE':
                self::atualizaTarefa($nrorg, $cdfilial, $nrtarefa, $dsitcronoprj);
                break;
            case 'ATIV':
                $planativtare = self::atualizaAtividade($nrorg, $cdfilial, $nrtarefa, $dsitcronoprj, $dadosatividade, $onlychangeoper);
                if ($planativtare) {
                    if(!$onlychangeoper) {
                        $cdprecitcrpj = $dadosatividade["CDPRECITCRPJ"];
                        $qtdiafolgacron = $dadosatividade["QTDIAFOLGACRON"];
                        $nrdiasitcrpj = $dadosatividade["NRDIASITCRPJ"];
                        if($nrdiasitcrpj == 0) {
                            $nrdiasitcrpj = $planativtare->getQttemplaitar();
                        }
                    }
                    
                    $cdoperador = $dadosatividade["CDOPERADOR"];
                    $cdgrupocargo = $dadosatividade["CDGRUPOCARGO"];
                    self::insereEnvolvidoProjeto($nrorg, $cdfilial, $cdprojeto, $cdoperador, $cdgrupocargo);
                }
                break;
        } 
        
        try {
            $cronoProjeto = \Teknisa\Entities\Cronoprojeto::findOneByCdfilialAndCdprojetoAndNrbaselineprAndCdvercronprjAndCdseqcronprjAndNrorg($cdfilial, $cdprojeto, $nrbaselinepr, $cdvercronprj, $cdseqcronprj, $nrorg);
                    
            if (is_object($cronoProjeto)) {
                if ($cdoperador != null) {
                    $cronoProjeto->setCdoperador($cdoperador);
                }
                if(!$onlychangeoper) {
                    $cronoProjeto->setDsitcronoprj($dsitcronoprj);
                    $cronoProjeto->setCdfilaprio($cdfilaprio);
                    $cronoProjeto->setCdprecitcrpj($cdprecitcrpj);
                    $cronoProjeto->setQtdiafolgacron($qtdiafolgacron);
                    $cronoProjeto->setNrdiasitcrpj($nrdiasitcrpj);
                }
                self::setEditAuditValues($cronoProjeto);
                $cronoProjeto->persist();
            }
           
            $connection->flush();
            
            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollback();
            throw $e;
        }
            
        return is_object($cronoProjeto) ? $cronoProjeto : false;
    }

    /**
     * Atualiza uma solicitação de serviço
     *
     * @param integer $nrOrg           Numero da organizacao.
     *
     * @return Soliserv
     *
     * @author newtonsantos
     */
    public static function atualizaSolicitacaoServico($nrorg, $cdfilial, $nrsoliserv, $cdprojeto, $dsrefesoli, $cdcliente, $nrlicencauso) {

        $dssoliserv = tk_get_message_by_name('msg_win_gep00001_000_DescricaoAberturaSolicitacao', array($cdprojeto, $dsrefesoli));
        
        $connection = \Setforms2\Shared\Registry::getEntityManager();
        try {
            $soliServ = \Faturamento\Entities\Soliserv::findOneByCdfilialAndNrsoliservAndNrorg($cdfilial, $nrsoliserv, $nrorg);
            
            if (is_object($soliServ)) {
                $soliServ->setDsrefesoli($dsrefesoli);
                $soliServ->setCdcliente($cdcliente);
                $soliServ->setNrlicencauso($nrlicencauso);
                self::setEditAuditValues($soliServ);
                $soliServ->persist();
            }
            
            $descSoli = \Teknisa\Entities\Descsoli::findOneByCdfilialAndNrsoliservAndNrorg($cdfilial, $nrsoliserv, $nrorg);
            
            if (is_object($descSoli)) {
                $descSoli->setDssoliserv($dssoliserv);
                self::setEditAuditValues($descSoli);
                $descSoli->persist();
            }
            
            $connection->flush();
        } catch (\Exception $e) {
            throw $e;
        }
            
        return is_object($soliServ) ? $soliServ : false;
    }

    /**
     * Insere uma tarefa
     *
     * @param integer $nrOrg           Numero da organizacao.
     *
     * @return Tarefa
     *
     * @author newtonsantos
     */
    public static function atualizaTarefa($nrorg, $cdfilial, $nrtarefa, $dsrefetarefa) {

        $connection = \Setforms2\Shared\Registry::getEntityManager();
        try {
            $tarefa = \Geral\Entities\Tarefa::findOneByCdfilialAndNrtarefaAndNrorg($cdfilial, $nrtarefa, $nrorg);
            
            if (is_object($tarefa)) {
                $tarefa->setDsrefetarefa($dsrefetarefa);
                self::setEditAuditValues($tarefa);
                $tarefa->persist();
            }
            
            $connection->flush();
        } catch (\Exception $e) {
            throw $e;
        }
            
        return is_object($tarefa) ? $tarefa : false;
    }

    /**
     * Atualiza uma atividade
     *
     * @param integer $nrOrg           Numero da organizacao.
     *
     * @return Planativtare
     *
     * @author newtonsantos
     */
    public static function atualizaAtividade($nrorg, $cdfilial, $nrtarefa, $dsitcronoprj, $dadosatividade, $onlychangeoper = false) {

        $nrplativtare = $dadosatividade["NRPLATIVTARE"];
        
        $platvrelittr = \Teknisa\Entities\Platvrelittr::findOneByCdfilialAndNrtarefaAndNrplativtareAndNrorg($cdfilial, $nrtarefa, $nrplativtare, $nrorg); 
        $nrsequittare = $platvrelittr->getNrsequittare();
        
        $cdoperador = $dadosatividade["CDOPERADOR"];
        if(!$onlychangeoper) {
            $cditemtrab = $dadosatividade["CDITEMTRAB"];
            $nrcplxittrab = $dadosatividade["NRCPLXITTRAB"];
            $nrdiasitcrpj = $dadosatividade["NRDIASITCRPJ"];
        }

        $connection = \Setforms2\Shared\Registry::getEntityManager();
        try {
            $itemtarefa = \Teknisa\Entities\Itemtarefa::findOneByCdfilialAndNrtarefaAndNrsequittareAndNrorg($cdfilial, $nrtarefa, $nrsequittare, $nrorg);
            
            if (is_object($itemtarefa)) {
                $itemtarefa->setCdoperador($cdoperador);
                if(!$onlychangeoper) {
                    $itemtarefa->setDsrefeittare($dsitcronoprj);
                    $itemtarefa->setCditemtrab($cditemtrab);
                    $itemtarefa->setNrcplxittare($nrcplxittrab);
                    $itemtarefa->setNrcplxitemtb($nrcplxittrab);
                }
                self::setEditAuditValues($itemtarefa);
                $itemtarefa->persist();
            }
            
            if(!$onlychangeoper) {
                $cdprocesso = $dadosatividade["CDPROCESSO"];
           
                $complexidade = self::getComplexAtividade($itemtarefa->getNrcplxittare(), $dadosatividade["VRFATOCPLX"], $dadosatividade["VRPRODMEDES"], $dadosatividade["QTTEMPMINATIV"], $dadosatividade["VRPESOATIVGRC"]);
                $nrcplxitativ = $complexidade["nrcplx"];
                $nrcplxplanat = $nrcplxitativ;
                $qttemplaitar = $complexidade["qttemp"];
                if(($nrdiasitcrpj != 0) && ($nrdiasitcrpj != $qttemplaitar)) {
                    $qttemplaitar = $nrdiasitcrpj;
                    $idatvtempaltcr = 'S';
                } else {
                    $qttemplaitar = $complexidade["qttemp"];
                    $idatvtempaltcr = 'N';
                }
            }
            
            $planativtare = \Geral\Entities\Planativtare::findOneByCdfilialAndNrtarefaAndNrplativtareAndNrorg($cdfilial, $nrtarefa, $nrplativtare, $nrorg);
            
            if (is_object($planativtare)) {
                $planativtare->setCdoperresp($cdoperador);
                if(!$onlychangeoper) {
                    $planativtare->setCdprocesso($cdprocesso);
                    $planativtare->setCditemtrab($cditemtrab);
                    $planativtare->setNrcplxitativ($nrcplxitativ);
                    $planativtare->setNrcplxplanat($nrcplxplanat);
                    $planativtare->setQttemplaitar($qttemplaitar);
                    $planativtare->setIdatvtempaltcr($idatvtempaltcr);
                }
                self::setEditAuditValues($planativtare);
                $planativtare->persist();
            }
            
            $connection->flush();
        } catch (\Exception $e) {
            throw $e;
        }
            
        return is_object($planativtare) ? $planativtare : false;
    }
    
    private static function setEditAuditValues($entity) {
        $nrorgatual = \Geral\Util\Ambiente::getNrorg(); 
        $dtatual = \Geral\Util\DateUtil::getDataAtual();
        $cdoperatual = \Geral\Util\Ambiente::getCdOperador();
        
        $entity->setDtultatu($dtatual);
        $entity->setNrorgultatu($nrorgatual);
        $entity->setCdoperultatu($cdoperatual);
        
        $dtinclusao = $entity->getDtinclusao();
        if($dtinclusao == null) {
            $entity->setDtinclusao($dtatual);
            $entity->setNrorginclusao($nrorgatual);
            $entity->setCdoperinclusao($cdoperatual);
        }
    }

    /**
     * Deleta um ítem de cronograma
     *
     * @param integer $nrOrg           Numero da organizacao.
     *
     * @return Cronoprojeto
     *
     * @author newtonsantos
     */
    public static function deletaItemCronogramaProjeto($nrorg, $cdfilial, $cdprojeto, $cdoperesprj, $nrbaselinepr, $cdvercronprj, $cdseqcronprj, $idnivelitcrono, $nrsoliserv, $nrtarefa, $nrplativtare) {
        $cronoprojetoRepository = \Teknisa\Entities\Cronoprojeto::getRepository();
        $cronoprojeto = $cronoprojetoRepository->consultaPercConclusaoCronograma($nrorg, $cdfilial, $cdprojeto, $nrbaselinepr, $cdvercronprj);

        $vrperconcprj = (count($cronoprojeto) > 0) ? $cronoprojeto[0]['VRPERCONCPRJ'] : 0;
        
        switch ($idnivelitcrono) {
            case 'SOLI':
                $where = 'CDFILIAL = ? AND CDPROJETO = ? AND NRBASELINEPR = ? AND CDVERCRONPRJ = ? AND NRSOLISERV = ? AND NRORG = ?';
                $params = array($cdfilial, $cdprojeto, $nrbaselinepr, $cdvercronprj, $nrsoliserv, $nrorg);
                break;
            case 'TARE':
                $where = 'CDFILIAL = ? AND CDPROJETO = ? AND NRBASELINEPR = ? AND CDVERCRONPRJ = ? AND NRSOLISERV = ? AND NRTAREFA = ? AND NRORG = ?';
                $params = array($cdfilial, $cdprojeto, $nrbaselinepr, $cdvercronprj, $nrsoliserv, $nrtarefa, $nrorg);
                break;
            case 'ATIV':
                $where = 'CDFILIAL = ? AND CDPROJETO = ? AND NRBASELINEPR = ? AND CDVERCRONPRJ = ? AND CDSEQCRONPRJ = ? AND NRORG = ?';
                $params = array($cdfilial, $cdprojeto, $nrbaselinepr, $cdvercronprj, $cdseqcronprj, $nrorg);
                break;
        }
        $order_by = 'NRSOLISERV, NRTAREFA, NRPLATIVTARE';
        
        $lstcronoprojeto = \Teknisa\Entities\Cronoprojeto::getObjectsByWhere($where, $params, $order_by);
        
        $connection = \Setforms2\Shared\Registry::getEntityManager();
        $connection->beginTransaction();
        try {
            foreach ($lstcronoprojeto as $cronoprojeto) {
                $nrsoliserv = $cronoprojeto->getNrsoliserv();
                $nrtarefa = $cronoprojeto->getNrtarefa();
                $nrplativtare = $cronoprojeto->getNrplativtare();
                $nrseqcronlst = $cronoprojeto->getCdseqcronprj();

                /* Um ítem de cronograma somente poderá ser excluído se 
                 * nenhuma atividade do cronograma tiver sido iniciada. */
                if($vrperconcprj == 0) {
                    self::deletaItemCronograma($nrorg, $cdfilial, $cdprojeto, $nrbaselinepr, $cdvercronprj, $nrseqcronlst);
                }

                if ($nrplativtare != null) {
                    self::deletaItemTarefa($nrorg, $cdfilial, $nrtarefa, $nrplativtare, $vrperconcprj);
                } elseif ($nrtarefa != null) {
                    self::deletaTarefa($nrorg, $cdfilial, $nrtarefa, $vrperconcprj);
                } else {
                    self::deletaSoliserv($nrorg, $cdfilial, $cdprojeto, $cdoperesprj, $nrbaselinepr, $nrsoliserv, $vrperconcprj);
                }
            }

            $connection->commit();
            
            tk_reload_data_with_filter('tab_gep00001_000_001', 'win_gep00001_000');
        } catch (\Exception $e) {
            $connection->rollback();

            throw $e;
        }
    }
    
    /**
     * Deleta um ítem de cronograma
     *
     * @param integer $nrOrg           Numero da organizacao.
     *
     * @return Cronoprojeto
     *
     * @author newtonsantos
     */
    public static function deletaItemCronograma($nrorg, $cdfilial, $cdprojeto, $nrbaselinepr, $cdvercronprj, $cdseqcronprj) {
        $connection = \Setforms2\Shared\Registry::getEntityManager();
        try {
            $cronoProjeto = \Teknisa\Entities\Cronoprojeto::findOneByCdfilialAndCdprojetoAndNrbaselineprAndCdvercronprjAndCdseqcronprjAndNrorg($cdfilial, $cdprojeto, $nrbaselinepr, $cdvercronprj, $cdseqcronprj, $nrorg); 
            if(is_object($cronoProjeto)) {
                $cronoProjeto->delete();
            }
            
            $connection->flush();
        } catch (\Exception $e) {
            throw $e;
        }
        
    }

    /**
     * Deleta um ítem de tarefa
     *
     * @param integer $nrOrg           Numero da organizacao.
     *
     * @return Cronoprojeto
     *
     * @author newtonsantos
     */
    public static function deletaItemTarefa($nrorg, $cdfilial, $nrtarefa, $nrplativtare, $vrperconcprj) {
        $connection = \Setforms2\Shared\Registry::getEntityManager();
        try {
            /* O item de tarefa somente será excluído caso nenhuma atividade 
             * tenha sido iniciada. Caso contrário, o item de tarefa será 
             * sinalizado como cancelado (PLANATIVTARE.IDSITATIVTAR = 'X').
             */
            if($vrperconcprj == 0) {
                $platvrelittr = \Teknisa\Entities\Platvrelittr::findOneByCdfilialAndNrtarefaAndNrplativtareAndNrorg($cdfilial, $nrtarefa, $nrplativtare, $nrorg); 
                if(is_object($platvrelittr)) {
                    $nrsequittare = $platvrelittr->getNrsequittare();
                    $platvrelittr->delete();
                }

                $planativtare = \Geral\Entities\Planativtare::findOneByCdfilialAndNrtarefaAndNrplativtareAndNrorg($cdfilial, $nrtarefa, $nrplativtare, $nrorg); 
                if(is_object($planativtare)) {
                    $planativtare->delete();
                }

                $itemtarefa = \Teknisa\Entities\Itemtarefa::findOneByCdfilialAndNrtarefaAndNrsequittareAndNrorg($cdfilial, $nrtarefa, $nrsequittare, $nrorg); 
                if(is_object($itemtarefa)) {
                    $itemtarefa->delete();
                }
            } else {
                $planativtare = \Geral\Entities\Planativtare::findOneByCdfilialAndNrtarefaAndNrplativtareAndNrorg($cdfilial, $nrtarefa, $nrplativtare, $nrorg);
                if(is_object($planativtare)) {
                    $idsitativtar = $planativtare->getIdsitativtar();

                    if($idsitativtar != 'C') {
                        $planativtare->setIdsitativtar('X');
                        $planativtare->persist();
                    }
                }
            }
            
            $connection->flush();
        } catch (\Exception $e) {
            throw $e;
        }
        
    }

    /**
     * Deleta uma tarefa
     *
     * @param integer $nrOrg           Numero da organizacao.
     *
     * @return Cronoprojeto
     *
     * @author newtonsantos
     */
    public static function deletaTarefa($nrorg, $cdfilial, $nrtarefa, $vrperconcprj) {
        $connection = \Setforms2\Shared\Registry::getEntityManager();
        try {
            /* A tarefa somente será excluída caso nenhuma atividade tenha 
             * sido iniciada. Caso contrário, a tarefa será sinalizada como
             * cancelada (IDSITTAREFA = 'X').
             */
            $tarefa = \Geral\Entities\Tarefa::findOneByCdfilialAndNrtarefaAndNrorg($cdfilial, $nrtarefa, $nrorg); 
            if(is_object($tarefa)) {
                if($vrperconcprj == 0) {
                    $tarefa->delete();
                } else {
                    $tarefa->setIdsittarefa('X');
                    $tarefa->persist();
                }
            }
            
            $connection->flush();
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * Deleta uma solicitação de serviço
     *
     * @param integer $nrOrg           Numero da organizacao.
     *
     * @return Cronoprojeto
     *
     * @author newtonsantos
     */
    public static function deletaSoliserv($nrorg, $cdfilial, $cdprojeto, $cdoperesprj, $nrbaselinepr, $nrsoliserv) {
        $paramgeral = \Geral\Entities\Paramgeral::findOneByNrorg($nrorg);
        $cdfatramence = $paramgeral->getCdfatramence();
        
        $dtatual = \Geral\Util\DateUtil::getDataAtual();
        $cdoperatual = \Geral\Util\Ambiente::getCdOperador();
        
        $txacaolftram = tk_get_message_by_name('msg_win_gep00001_000_FaseDeEncerramentoSolicitacao');
        $dsalfasetram = tk_get_message_by_name('msg_win_gep00001_000_DescricaoEncerramentoSolicitacao', array($cdprojeto, $cdoperatual));
            
        $connection = \Setforms2\Shared\Registry::getEntityManager();
        try {
            /* Ao excluir uma solicitação, ela será
             * apenas tramitada para seu encerramento.
             */
            $logFaseTram = new \Teknisa\Entities\Logfasetram();
            $logFaseTram->setNrorg($nrorg);
            $logFaseTram->setCdfilial($cdfilial);
            $logFaseTram->setNrsoliserv($nrsoliserv);
            $logFaseTram->setDtalfasetram($dtatual);
            $logFaseTram->setDsalfasetram($dsalfasetram);
            $logFaseTram->setCdfasetram($cdfatramence);
            $logFaseTram->setCdoperador($cdoperatual);
            $logFaseTram->setCdoperresp($cdoperesprj);
            $logFaseTram->setTxacaolftram($txacaolftram);
            $logFaseTram->setDtinclusao($dtatual);
            $logFaseTram->setDtultatu($dtatual);
            $logFaseTram->setNrorginclusao($nrorg);
            $logFaseTram->setCdoperinclusao($cdoperatual);
            $logFaseTram->setNrorgultatu($nrorg);
            $logFaseTram->setCdoperultatu($cdoperatual);
            $logFaseTram->persist();

            $soliBaseLine = \Teknisa\Entities\Solibaseline::findOneByCdfilialAndCdprojetoAndNrbaselineprAndNrsoliservAndNrorg($cdfilial, $cdprojeto, $nrbaselinepr, $nrsoliserv, $nrorg); 
            if(is_object($soliBaseLine)) {
                $soliBaseLine->setIdsitusolbaseli('E');
                $soliBaseLine->persist();
            }

            $soliServ = \Faturamento\Entities\Soliserv::findOneByCdfilialAndNrsoliservAndNrorg($cdfilial, $nrsoliserv, $nrorg); 
            if(is_object($soliServ)) {
                $soliServ->setCdfasetram($cdfatramence);
                $soliServ->setDtencesoli($dtatual);
                $soliServ->setIdstatussoli('C');
                $soliServ->setIdsolience('S');
                $soliServ->setCdoperresp($cdoperesprj);
                $soliServ->persist();
            }
            
            $connection->flush();
        } catch (\Exception $e) {
            throw $e;
        }
        
    }
    
    /**
     * Rotina para organização e gravação da fila das atividades.
     *
     * @param integer $nrOrg           Numero da organizacao.
     *
     * @author newtonsantos
     */
    public static function organizaFilaCronograma($nrorg, $cdfilial, $cdprojeto, $nrbaselinepr, $cdvercronprj) {
        $sol_where = 'CDFILIAL = ? AND CDPROJETO = ? AND NRBASELINEPR = ? AND CDVERCRONPRJ = ? AND NRORG = ? AND CDNIVELITPRJ = ?';
        $sol_params = array($cdfilial, $cdprojeto, $nrbaselinepr, $cdvercronprj, $nrorg, '2');
        $sol_order_by = "NVL(CDFILAPRIO,'000'), NVL(NRSOLISERV,'0000000001'), NVL(NRTAREFA,'0000000000'), TO_NUMBER(NVL(CDSEQCRONPRJ,0))";
        
        $soliserv = \Teknisa\Entities\Cronoprojeto::getObjectsByWhere($sol_where, $sol_params, $sol_order_by);

        $nrordsoli = 0;
        $ordsoliserv = array();
        foreach ($soliserv as $cronoprojeto) {
            $nrordsoli += 1;
            $stordsoli = \Geral\Util\Util::zeroFill($nrordsoli, 3);
            $ordsoliserv[$cronoprojeto->getNrsoliserv()] = $stordsoli;
        }
        
        $tar_where = 'CDFILIAL = ? AND CDPROJETO = ? AND NRBASELINEPR = ? AND CDVERCRONPRJ = ? AND NRORG = ? AND CDNIVELITPRJ = ?';
        $tar_params = array($cdfilial, $cdprojeto, $nrbaselinepr, $cdvercronprj, $nrorg, 'T');
        $tar_order_by = "NVL(CDFILAPRIO,'000'), NVL(NRSOLISERV,'0000000001'), NVL(NRTAREFA,'0000000000'), TO_NUMBER(NVL(CDSEQCRONPRJ,0))";
        
        $tarefa = \Teknisa\Entities\Cronoprojeto::getObjectsByWhere($tar_where, $tar_params, $tar_order_by);

        $nrordtare = 0;
        $ordtarefa = array();
        foreach ($tarefa as $cronoprojeto) {
            $nrordtare += 1;
            $stordtare = \Geral\Util\Util::zeroFill($nrordtare, 4);
            $ordtarefa[$cronoprojeto->getNrtarefa()] = $ordsoliserv[$cronoprojeto->getNrsoliserv()].$stordtare;
        }
        
        $cpj_where = 'CDFILIAL = ? AND CDPROJETO = ? AND NRBASELINEPR = ? AND CDVERCRONPRJ = ? AND NRORG = ? AND NOT (NRSOLISERV IS NULL)';
        $cpj_params = array($cdfilial, $cdprojeto, $nrbaselinepr, $cdvercronprj, $nrorg);
        $cpj_order_by = "NVL(CDFILAPRIO,'000'), NVL(NRSOLISERV,'0000000001'), NVL(NRTAREFA,'0000000000'), TO_NUMBER(NVL(CDSEQCRONPRJ,0))";
        
        $allcronoprojeto = \Teknisa\Entities\Cronoprojeto::getObjectsByWhere($cpj_where, $cpj_params, $cpj_order_by);
        
        $ordcronoproj = array();
        foreach ($allcronoprojeto as $cronoprojeto) {
            switch ($cronoprojeto->getCdnivelitprj()) {
                case '2':
                    $nrsoliserv = $cronoprojeto->getNrsoliserv();
                    $stordcrono = $ordsoliserv[$nrsoliserv].'0000000'.'00000';
                    break;
                case 'T':
                    $nrsoliserv = $cronoprojeto->getNrsoliserv();
                    $nrtarefa = $cronoprojeto->getNrtarefa();
                    $stordcrono = $ordsoliserv[$nrsoliserv].$ordtarefa[$nrtarefa].'00000';
                    break;
                case '3':
                    $nrsoliserv = $cronoprojeto->getNrsoliserv();
                    $nrtarefa = $cronoprojeto->getNrtarefa();
                    $cdfilaorseq = $cronoprojeto->getCdfilaprio() ? $cronoprojeto->getCdfilaprio() : $cronoprojeto->getCdseqcronprj();
                    $cdfilaprio = \Geral\Util\Util::zeroFill($cdfilaorseq, 5);
                    $stordcrono = $ordsoliserv[$nrsoliserv].$ordtarefa[$nrtarefa].$cdfilaprio;
                    break;
            }
            $ordcronoproj[] = array("CRONO" => $cronoprojeto, "ORDEM" => $stordcrono);
        }
        
        $lstcronoprojeto = self::ordenaArray($ordcronoproj, 'ORDEM');
        
        $connection = \Setforms2\Shared\Registry::getEntityManager();
        try {
            $nrfila = 1;
            foreach ($lstcronoprojeto as $ordcrpj) {
                $hexfilaprio = strtoupper(dechex($nrfila));
                $cdfilaprio = \Geral\Util\Util::zeroFill($hexfilaprio, 3);

                $cronoprojeto = $ordcrpj["CRONO"];
                $cronoprojeto->setCdfilaprio($cdfilaprio);
                $cronoprojeto->persist();

                $nrfila += 3;
            }
            
            $connection->flush();
        } catch (\Exception $e) {
            throw $e;
        }

        tk_reload_data_with_filter('tab_gep00001_000_001', 'win_gep00001_000');
    }
    
    /**
     * Rotina para cálculo das datas inicial e final de cada uma das atividades.
     *
     * @param integer $nrOrg           Numero da organizacao.
     *
     * @author newtonsantos
     */
    public static function calculaDatasCronograma($nrorg, $cdfilial, $cdprojeto, $nrbaselinepr, $cdvercronprj, $dtinicial) {
        try {
            $procedure = new Procedure("CALCULA_CRONOGRAMA");
            $procedure->addParam(new Param(
                "P_CDFILIAL",
                Param::PARAM_INPUT,
                $cdfilial,
                Param::PARAM_TYPE_STR
            ));
            $procedure->addParam(new Param(
                "P_CDPROJETO",
                Param::PARAM_INPUT,
                $cdprojeto,
                Param::PARAM_TYPE_STR
            ));
            $procedure->addParam(new Param(
                "P_NRBASELINEPR",
                Param::PARAM_INPUT,
                $nrbaselinepr,
                Param::PARAM_TYPE_STR
            ));
            $procedure->addParam(new Param(
                "P_CDVERCRONPRJ",
                Param::PARAM_INPUT,
                $cdvercronprj,
                Param::PARAM_TYPE_STR
            ));
            $procedure->addParam(new Param(
                "P_DTDATABASE",
                Param::PARAM_INPUT,
                $dtinicial,
                Param::PARAM_TYPE_STR
            ));
            $procedure->call();
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    public static function executaTrocaDeFila($nrorg, $cdfilial, $cdprojeto, $nrbaselinepr, $cdvercronprj, 
                                              $cdseqcronprj, $dsitcronoprj, $idnivelitcrono, $cdcliente,
                                              $nrlicencauso, $cdfilaprio, $nrsoliserv, $nrtarefa, $dadosatividade) {
        
        $cronoprojtomove = tk_get_context_vars('CRONOPROJTOMOVE', 'win_gep00001_000');
        
        if($cronoprojtomove) {
            $nrorg_tomove = $cronoprojtomove['nrorg'];
            $cdfilial_tomove = $cronoprojtomove['cdfilial'];
            $cdprojeto_tomove = $cronoprojtomove['cdprojeto'];
            $nrbaselinepr_tomove = $cronoprojtomove['nrbaselinepr'];
            $cdvercronprj_tomove = $cronoprojtomove['cdvercronprj'];
            $cdseqcronprj_tomove = $cronoprojtomove['cdseqcronprj'];
            $dsitcronoprj_tomove = $cronoprojtomove['dsitcronoprj'];
            $idnivelitcrono_tomove = $cronoprojtomove['idnivelitcrono'];
            $cdcliente_tomove = $cronoprojtomove['cdcliente'];
            $nrlicencauso_tomove = $cronoprojtomove['nrlicencauso'];
            $nrsoliserv_tomove = $cronoprojtomove['nrsoliserv'];
            $nrtarefa_tomove = $cronoprojtomove['nrtarefa'];
            $dadosatividade_tomove = $cronoprojtomove['dadosatividade'];
            
            $idnivelit_atu = $idnivelitcrono;
            $nrtarefa_atu = $nrtarefa;

            if($idnivelit_atu != $idnivelitcrono_tomove) {
                tk_show_message_by_name('msg_win_gep00001_000_MoveOutroTipoBloq', array($cdseqcronprj_tomove, $dsitcronoprj_tomove), 'warning');
            } elseif(($idnivelit_atu == 'ATIV') && ($nrtarefa_atu != $nrtarefa_tomove)) {
                tk_show_message_by_name('msg_win_gep00001_000_MoveAtivOutraTarefaBloq', array(), 'warning');
            } else {
                $nrfilaprio = hexdec($cdfilaprio)-1;
                $hexfilaprio = strtoupper(dechex($nrfilaprio));
                $cdfilaprio_tomove = \Geral\Util\Util::zeroFill($hexfilaprio, 3);
            
                tk_show_message_by_name('msg_win_gep00001_000_MovendoItemCronograma', array($cdseqcronprj_tomove, $dsitcronoprj_tomove));
                self::atualizaItemCronogramaProjeto($nrorg_tomove, $cdfilial_tomove, $cdprojeto_tomove, $nrbaselinepr_tomove, $cdvercronprj_tomove, 
                                                    $cdseqcronprj_tomove, $dsitcronoprj_tomove, $idnivelitcrono_tomove, $cdcliente_tomove, 
                                                    $nrlicencauso_tomove, $cdfilaprio_tomove, $nrsoliserv_tomove, $nrtarefa_tomove, $dadosatividade_tomove);

                tk_show_message_by_name('msg_win_gep00001_000_OrganizandoFilaCronograma');
                \Geral\Logic\Projeto\Projeto::organizaFilaCronograma($nrorg, $cdfilial, $cdprojeto, $nrbaselinepr, $cdvercronprj);
            }
            tk_set_context_vars('CRONOPROJTOMOVE', 'win_gep00001_000', null);
            
        } else {
            
            $cronoprojtomove = array(
                'nrorg'          => $nrorg,
                'cdfilial'       => $cdfilial,
                'cdprojeto'      => $cdprojeto,
                'nrbaselinepr'   => $nrbaselinepr,
                'cdvercronprj'   => $cdvercronprj,
                'cdseqcronprj'   => $cdseqcronprj,
                'dsitcronoprj'   => $dsitcronoprj,
                'idnivelitcrono' => $idnivelitcrono,
                'cdcliente'      => $cdcliente,
                'nrlicencauso'   => $nrlicencauso,
                'cdfilaprio'     => $cdfilaprio,
                'nrsoliserv'     => $nrsoliserv,
                'nrtarefa'       => $nrtarefa,
                'dadosatividade' => $dadosatividade
            );
            
            tk_set_context_vars('CRONOPROJTOMOVE', 'win_gep00001_000', $cronoprojtomove);
            
            tk_show_message_by_name('msg_win_gep00001_000_ItemCronoMoveFila', array($cdseqcronprj, $dsitcronoprj));
        }
    }

    /**
     * Troca operador responsável pelas atividades automaticamente de acordo com parâmetros informados pelo usuário
     *
     * @param integer $nrOrg           Número da organização.
     *
     * @author newtonsantos
     */
    public static function trocaAutomaticaOperadores($nrorg, $cdfilial, $cdprojeto, $nrbaselinepr, $cdvercronprj, $nrsoliserv, $nrtarefa, $cdoperador) {
        $cdnivelitprj = '3';       /* A troca de operador será realizada apenas nas atividades do */
        $idnivelitcrono = 'ATIV';  /* cronograma (cdnivelitprj = "3" e idnivelitcrono = "ATIV").  */
        
        $operador = \Geral\Entities\Operador::findOneByCdoperadorAndNrorg($cdoperador, $nrorg); 
        $cargo = \Geral\Entities\Cargo::findOneByCdcargoAndNrorg($operador->getCdcargo(), $nrorg); 
        
        $where = 'CDFILIAL = ? AND CDPROJETO = ? AND NRBASELINEPR = ? AND CDVERCRONPRJ = ? AND CDNIVELITPRJ = ? AND NRORG = ?';
        $params = array($cdfilial, $cdprojeto, $nrbaselinepr, $cdvercronprj, $cdnivelitprj, $nrorg);
        if($nrsoliserv != 'TODAS') {
            $where .= ' AND NRSOLISERV = ?';
            $params[] = $nrsoliserv;
        }
        if($nrtarefa != 'TODAS') {
            $where .= ' AND NRTAREFA = ?';
            $params[] = $nrtarefa;
        }
        $order_by = 'NRSOLISERV, CDFILAPRIO';
        $lstcronoprojeto = \Teknisa\Entities\Cronoprojeto::getObjectsByWhere($where, $params, $order_by);
        
        try {
            $nrsoliaux = '';
            foreach ($lstcronoprojeto as $cronoprojeto) {
    
                $cdseqcronprj = $cronoprojeto->getCdseqcronprj();
                $dsitcronoprj = $cronoprojeto->getDsitcronoprj();
                $nrsoliserv = $cronoprojeto->getNrsoliserv();
                $nrtarefa = $cronoprojeto->getNrtarefa();
                $cdoperatual = $cronoprojeto->getCdoperador();
                
                if($cdoperatual != $cdoperador) {
                    
                    if($nrsoliaux != $nrsoliserv) {
                        $soliserv = \Faturamento\Entities\Soliserv::findOneByCdfilialAndNrsoliservAndNrorg($cdfilial, $nrsoliserv, $nrorg);             

                        $cdcliente = $soliserv->getCdcliente();
                        $nrlicencauso = $soliserv->getNrlicencauso();

                        $nrsoliaux = $nrsoliserv;
                    }

                    $nrplativtare = $cronoprojeto->getNrplativtare();
                    $cditemtrab = '';
                    $nrcplxittrab = '';
                    $cdprocesso = '';
                    $vrfatocplx = '';
                    $qttempminativ = '';
                    $vrprodmedes = '';
                    $cdfilaprio = '';
                    $cdprecitcrpj = '';
                    $qtdiafolgacron = '';
                    $nrdiasitcrpj = '';
                    $cdgrupocargo = $cargo->getCdgrupocargo();
                    $vrpesoativgrc = '';

                    $dadosatividade = array(
                        "NRPLATIVTARE"   => $nrplativtare,
                        "CDITEMTRAB"     => $cditemtrab,
                        "NRCPLXITTRAB"   => $nrcplxittrab,
                        "CDPROCESSO"     => $cdprocesso,
                        "VRFATOCPLX"     => $vrfatocplx,
                        "QTTEMPMINATIV"  => $qttempminativ,
                        "VRPRODMEDES"    => $vrprodmedes,
                        "CDOPERADOR"     => $cdoperador,
                        "CDFILAPRIO"     => $cdfilaprio,
                        "CDPRECITCRPJ"   => $cdprecitcrpj,
                        "QTDIAFOLGACRON" => $qtdiafolgacron,
                        "NRDIASITCRPJ"   => $nrdiasitcrpj,
                        "CDGRUPOCARGO"   => $cdgrupocargo,
                        "VRPESOATIVGRC"  => $vrpesoativgrc,
                    );

                    $planativtare = \Geral\Entities\Planativtare::findOneByCdfilialAndNrtarefaAndNrplativtareAndNrorg($cdfilial, $nrtarefa, $nrplativtare, $nrorg);

                    $dotrocaoper = true;
                    if(is_object($planativtare)) {
                        $vrperconcprj = $cronoprojeto->getVrperconcprj();
                        $idsitativtar = $planativtare->getIdsitativtar();

                        if(($vrperconcprj == 100) || !(strpos('CX', $idsitativtar) === FALSE)) {
                            $dotrocaoper = false;
                        } 
                    }

                    if($dotrocaoper) {
                        $onlychangeoper = true;
                        self::atualizaItemCronogramaProjeto($nrorg, $cdfilial, $cdprojeto, $nrbaselinepr, $cdvercronprj, $cdseqcronprj, $dsitcronoprj, $idnivelitcrono, $cdcliente, $nrlicencauso, $cdfilaprio, $nrsoliserv, $nrtarefa, $dadosatividade, $onlychangeoper);
                    }
                }
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    /**
     * Rotina para geração de nova versão para um cronograma.
     *
     * @param integer $nrOrg           Numero da organizacao.
     *
     * @author newtonsantos
     */
    public static function geraNovaVersaoCronograma($nrorg, $cdfilial, $cdprojeto, $nrbaselinepr, $cdvercronprj) {
        $cronoprojetoRepository = \Teknisa\Entities\Cronoprojeto::getRepository();
        try{
            $cdnovaversao = $cronoprojetoRepository->geraNovaVersaoCronograma($nrorg, $cdfilial, $cdprojeto, $nrbaselinepr, $cdvercronprj);
            
            tk_refresh_ref_table('CDVERCRONPRJ', 'tab_gep00001_000_001_filter', 'win_gep00001_000');
            tk_set_context_vars('CDVERCRONPRJ-ATUAL', 'win_gep00001_000', 'CHECK');        
            tk_reload_data_with_filter('tab_gep00001_000_001', 'win_gep00001_000');
            
            tk_show_message_by_name('msg_win_gep00001_000_NovaVersaoCronogramaGerada', array($cdnovaversao));            
        } catch(\Exception $e) {
            throw $e;
        }
    }
    
    public static function retornaNmfilial($cdfilial,$nrorg){
        try{
            $filial = \Geral\Entities\Filial::retornaFilial($cdfilial, $nrorg);
        
            $nmfilial = $filial->getNmfilial();
            return $nmfilial;
        }
        catch(\Exception $e){
            
            return;
        }
    }
}

?>
